#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include<Servo.h>
#include<DHT.h>

#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);

#define servopin 8
Servo gate;

#define dhtPin 11
#define dhtType DHT11
DHT dht(dhtPin,dhtType);

#define trigRch1 10
#define trigRch2 9

#define handp 6

#define led 5

float tempe = 0;
float hum = 0;
float speedofSound = 0;
float speedofSoundCm = 0;

int insideCount = 0

float uin()
{
  float timein = 0;
  pinMode(trigRch1,OUTPUT);
  digitalWrite(trigRch1,LOW);
  delayMicroseconds(2);
  digitalWrite(trigRch1,HIGH);
  delayMicroseconds(10);
  digitalWrite(trigRch1,LOW);
  pinMode(trigRch1,INPUT);
  timein = pulseIn(trigRch1,HIGH);
  return timein;
}

float uout()
{
  float timeout = 0;
  pinMode(trigRch2,OUTPUT);
  digitalWrite(trigRch2,LOW);
  delayMicroseconds(2);
  digitalWrite(trigRch2,HIGH);
  delayMicroseconds(10);
  digitalWrite(trigRch2,LOW);
  pinMode(trigRch2,INPUT);
  timeout = pulseIn(trigRch2,HIGH);
  return timeout;
}

void setup()
{
  Serial.begin(115200);
  dht.begin();
  gate.attach(servopin);
  gate.write(70);
  pinMode(led,OUTPUT);
  pinMode(handp,INPUT);
  pinMode(led,OUTPUT);
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(1);
  display.setCursor(0,0);
  display.print("STARTING........");
  display.setCursor(0,10);
  display.print("PLEASE WAIT");
  display.display();
  delay(2000);
}
void loop()
{
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(1);
  display.setCursor(0,0);
  display.print("PEOPLES INSIDE");
  display.setCursor(0,10);
  display.print(insideCount);
  display.display();
  
  float enterDist = 0;
  float exitDist = 0;
  float ldr = 0;
  int handDist = 0;
  float inTime = 0;
  float outTime = 0;
  
  tempe = dht.readTemperature();
  hum = dht.readHumidity();
  speedofSound = 331.4+(0.606*tempe)+(0.0124*hum);
  speedofSoundCm = speedofSound/10000;

  inTime = uin();
  outTime = uout();
  
  enterDist = (inTime/2)*speedofSoundCm;
  exitDist = (outTime/2)*speedofSoundCm;
  handDist = digitalRead(handp);
  
  if(enterDist <= 10 && insideCount<5)
  {
    insideCount = insideCount + 1;
    gate.write(180);
    Serial.println("GATE OPENED");
    Serial.println(insideCount);
    delay(2000);
    gate.write(70);
    Serial.println("PLEASE SANITIZE YOUR HAND");
    Serial.println(insideCount);
    
    display.clearDisplay();
    display.setTextColor(WHITE);
    display.setTextSize(1);
    display.setCursor(0,0);
    display.print("WARNING..........");
    display.setCursor(0,10);
    display.print("PLEASE SANITIZE HAND");
    display.display();
    while(handDist == 1)
    {
      handDist = digitalRead(handp);
    }
    digitalWrite(led,HIGH);
    display.clearDisplay();
    display.setTextColor(WHITE);
    display.setTextSize(1);
    display.setCursor(0,0);
    display.print("DONT REMOVE HAND");
    display.setCursor(0,10);
    display.print("WHILE SANITIZING");
    display.display();
    
    delay(2000);
    digitalWrite(led,LOW);
    Serial.println("SANITIZATION DONE");
    Serial.println(insideCount);
    
    display.clearDisplay();
    display.setTextColor(WHITE);
    display.setTextSize(1);
    display.setCursor(0,0);
    display.print("PROCESS COMPLETE");
    display.setCursor(0,10);
    display.print("THANKS FOR USING");
    display.display();
    
    delay(1000);
    
  }

  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(1);
  display.setCursor(0,0);
  display.print("PEOPLES INSIDE");
  display.setCursor(0,10);
  display.print(insideCount);
  display.display();
  
  if(exitDist <= 10 && insideCount>0)
  {
    insideCount = insideCount - 1;
    gate.write(180);
    Serial.println("GATE OPENED");
    Serial.println(insideCount);
    delay(2000);
    gate.write(70);
    Serial.println("GATE CLOSED");
    Serial.println(insideCount);
  }
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(1);
  display.setCursor(0,0);
  display.print("PEOPLES INSIDE");
  display.setCursor(0,10);
  display.print(insideCount);
  display.display();  
}
